@@ -41,4 +41,5 @@ _directory  # unused attribute (autonomy/cli/hash.py:63)
 generate_all  # unused function (autonomy/cli/hash.py:72)
 hash_file  # unused function (autonomy/cli/hash.py:94)
 _create_or_update_from_json  # unused method (autonomy/configurations/base.py:175)
-help  # unused attribute  (60% confidence) # (autonomy/cli/core.py:41)
+help  # unused attribute (autonomy/cli/core.py:41)
+DEFAULT_ABCI_BUILD_DIR  # unused variable (autonomy/deploy/constants.py:105)
