@@ -18,37 +18,59 @@
 # ------------------------------------------------------------------------------
 
 """Integration tests for the valory/oracle_abci skill."""
+from copy import deepcopy
+from typing import Tuple
+
 import pytest
 from aea.configurations.data_types import PublicId
 
 from tests.fixture_helpers import UseGnosisSafeHardHatNet
-from tests.test_agents.base import (
-    BaseTestEnd2EndAgentCatchup,
-    BaseTestEnd2EndNormalExecution,
+from tests.test_agents.base import BaseTestEnd2EndExecution, RoundChecks
+
+
+HAPPY_PATH: Tuple[RoundChecks, ...] = (
+    RoundChecks("registration_startup"),
+    RoundChecks("randomness_safe"),
+    RoundChecks("select_keeper_safe"),
+    RoundChecks("deploy_safe"),
+    RoundChecks("validate_safe"),
+    RoundChecks("randomness_oracle"),
+    RoundChecks("select_keeper_oracle"),
+    RoundChecks("deploy_oracle"),
+    RoundChecks("validate_oracle"),
+    RoundChecks("estimate_consensus", n_periods=2),
+    RoundChecks("tx_hash", n_periods=2),
+    RoundChecks("randomness_transaction_submission", n_periods=2),
+    RoundChecks("select_keeper_transaction_submission_a", n_periods=2),
+    RoundChecks("collect_signature", n_periods=2),
+    RoundChecks("finalization", n_periods=2),
+    RoundChecks("validate_transaction", n_periods=2),
+    RoundChecks("reset_and_pause", n_periods=2),
+    RoundChecks("collect_observation", n_periods=3),
 )
 
 
-# round check log messages of the happy path
-EXPECTED_ROUND_LOG_COUNT = {
-    "registration_startup": 1,
-    "randomness_safe": 1,
-    "select_keeper_safe": 1,
-    "deploy_safe": 1,
-    "validate_safe": 1,
-    "randomness_oracle": 1,
-    "select_keeper_oracle": 1,
-    "deploy_oracle": 1,
-    "validate_oracle": 1,
-    "estimate_consensus": 2,
-    "tx_hash": 2,
-    "randomness_transaction_submission": 2,
-    "select_keeper_transaction_submission_a": 2,
-    "collect_signature": 2,
-    "finalization": 2,
-    "validate_transaction": 2,
-    "reset_and_pause": 2,
-    "collect_observation": 3,
-}
+def _generate_reset_happy_path(
+    n_resets_to_perform: int, reset_tendermint_every: int
+) -> Tuple[RoundChecks, ...]:
+    """Generate the happy path for checking the oracle combined with the Tendermint reset functionality."""
+    happy_path = deepcopy(HAPPY_PATH)
+    for round_checks in happy_path:
+        if round_checks.name in (
+            "estimate_consensus"
+            "tx_hash"
+            "randomness_transaction_submission"
+            "select_keeper_transaction_submission_a"
+            "collect_signature"
+            "finalization"
+            "validate_transaction"
+            "reset_and_pause"
+            "collect_observation"
+        ):
+            round_checks.n_periods = n_resets_to_perform * reset_tendermint_every
+
+    return happy_path
+
 
 # strict check log messages of the happy path
 STRICT_CHECK_STRINGS = (
@@ -62,7 +84,7 @@ STRICT_CHECK_STRINGS = (
 
 @pytest.mark.parametrize("nb_nodes", (1,))
 class TestABCIPriceEstimationSingleAgent(
-    BaseTestEnd2EndNormalExecution,
+    BaseTestEnd2EndExecution,
     UseGnosisSafeHardHatNet,
 ):
     """Test the ABCI oracle skill with only one agent."""
@@ -71,12 +93,12 @@ class TestABCIPriceEstimationSingleAgent(
     skill_package = "valory/oracle_abci:0.1.0"
     wait_to_finish = 180
     strict_check_strings = STRICT_CHECK_STRINGS
-    round_check_strings_to_n_periods = EXPECTED_ROUND_LOG_COUNT
+    happy_path = HAPPY_PATH
 
 
 @pytest.mark.parametrize("nb_nodes", (2,))
 class TestABCIPriceEstimationTwoAgents(
-    BaseTestEnd2EndNormalExecution,
+    BaseTestEnd2EndExecution,
     UseGnosisSafeHardHatNet,
 ):
     """Test the ABCI oracle skill with two agents."""
@@ -85,12 +107,12 @@ class TestABCIPriceEstimationTwoAgents(
     skill_package = "valory/oracle_abci:0.1.0"
     wait_to_finish = 180
     strict_check_strings = STRICT_CHECK_STRINGS
-    round_check_strings_to_n_periods = EXPECTED_ROUND_LOG_COUNT
+    happy_path = HAPPY_PATH
 
 
 @pytest.mark.parametrize("nb_nodes", (4,))
 class TestABCIPriceEstimationFourAgents(
-    BaseTestEnd2EndNormalExecution,
+    BaseTestEnd2EndExecution,
     UseGnosisSafeHardHatNet,
 ):
     """Test the ABCI oracle skill with four agents."""
@@ -99,20 +121,22 @@ class TestABCIPriceEstimationFourAgents(
     skill_package = "valory/oracle_abci:0.1.0"
     wait_to_finish = 180
     strict_check_strings = STRICT_CHECK_STRINGS
-    round_check_strings_to_n_periods = EXPECTED_ROUND_LOG_COUNT
+    happy_path = HAPPY_PATH
 
 
 @pytest.mark.parametrize("nb_nodes", (4,))
-class TestAgentCatchup(BaseTestEnd2EndAgentCatchup, UseGnosisSafeHardHatNet):
+class TestAgentCatchup(BaseTestEnd2EndExecution, UseGnosisSafeHardHatNet):
     """Test that an agent that is launched later can synchronize with the rest of the network"""
 
     agent_package = "valory/oracle:0.1.0"
     skill_package = "valory/oracle_abci:0.1.0"
     wait_to_finish = 200
     restart_after = 45
-    round_check_strings_to_n_periods = EXPECTED_ROUND_LOG_COUNT
+    happy_path = HAPPY_PATH
     stop_string = "'registration_startup' round is done with event: Event.DONE"
 
+    n_terminal = 1
+
 
 @pytest.mark.skip
 class TestTendermintReset(TestABCIPriceEstimationFourAgents):
@@ -121,28 +145,17 @@ class TestTendermintReset(TestABCIPriceEstimationFourAgents):
     skill_package = "valory/oracle_abci:0.1.0"
     wait_to_finish = 360
     # run for 4 periods instead of 2
-    round_check_strings_to_n_periods = {
-        round: 4
-        for round in EXPECTED_ROUND_LOG_COUNT.keys()
-        if round
-        in (
-            "estimate_consensus"
-            "tx_hash"
-            "randomness_transaction_submission"
-            "select_keeper_transaction_submission_a"
-            "collect_signature"
-            "finalization"
-            "validate_transaction"
-            "reset_and_pause"
-            "collect_observation"
-        )
-    }
+    __n_resets_to_perform = 2
+    __reset_tendermint_every = 2
+    happy_path = _generate_reset_happy_path(
+        __n_resets_to_perform, __reset_tendermint_every
+    )
     __args_prefix = f"vendor.valory.skills.{PublicId.from_str(skill_package).name}.models.params.args"
     # reset every two rounds
     extra_configs = [
         {
             "dotted_path": f"{__args_prefix}.reset_tendermint_after",
-            "value": 2,
+            "value": __reset_tendermint_every,
         },
         {
             "dotted_path": f"{__args_prefix}.observation_interval",
@@ -165,11 +178,9 @@ class TestTendermintResetInterrupt(TestAgentCatchup):
 
     # stop for restart_after seconds when resetting Tendermint for the first time (using -1 because count starts from 0)
     stop_string = f"Entered in the 'reset_and_pause' round for period {__reset_tendermint_every - 1}"
-    # check if we manage to reset with Tendermint `__n_resets_to_perform` times with the rest of the agents
-    exclude_from_checks = [3]
-    round_check_strings_to_n_periods = {
-        "reset_and_pause": __n_resets_to_perform * __reset_tendermint_every
-    }
+    happy_path = _generate_reset_happy_path(
+        __n_resets_to_perform, __reset_tendermint_every
+    )
     __args_prefix = f"vendor.valory.skills.{PublicId.from_str(skill_package).name}.models.params.args"
     # reset every `__reset_tendermint_every` rounds
     extra_configs = [
@@ -196,3 +207,5 @@ class TestTendermintResetInterruptNoRejoin(TestTendermintResetInterrupt):
     wait_to_finish = 300
     # set the restart to a value so that the agent never rejoins, in order to test the impact to the rest of the agents
     restart_after = wait_to_finish
+    # check if we manage to reset with Tendermint `__n_resets_to_perform` times with the rest of the agents
+    exclude_from_checks = [3]
