@@ -32,6 +32,7 @@ from packages.valory.skills.abstract_round_abci.base import (
     AbciAppDB,
     BaseTxPayload,
     MAX_INT_256,
+    TransactionNotValidError,
 )
 from packages.valory.skills.oracle_deployment_abci.payloads import RandomnessPayload
 from packages.valory.skills.transaction_settlement_abci.payload_tools import (
@@ -658,6 +659,28 @@ class TestSynchronizeLateMessagesRound(BaseCollectNonEmptyUntilThresholdRound):
             )
         )
 
+    def test_incorrect_serialization_not_accepted(self) -> None:
+        """Test wrong serialization not collected"""
+
+        test_round = SynchronizeLateMessagesRound(
+            synchronized_data=self.synchronized_data,
+            consensus_params=self.consensus_params,
+        )
+        sender = list(test_round.accepting_payloads_from).pop()
+        tx_hashes = "0" * (TX_HASH_LENGTH - 1)
+        payload = SynchronizeLateMessagesPayload(sender=sender, tx_hashes=tx_hashes)
+
+        with pytest.raises(
+            TransactionNotValidError, match="Expecting serialized data of chunk size"
+        ):
+            test_round.check_payload(payload)
+
+        with pytest.raises(
+            ABCIAppInternalError, match="Expecting serialized data of chunk size"
+        ):
+            test_round.process_payload(payload)
+        assert payload not in test_round.collection
+
 
 def test_synchronized_datas() -> None:
     """Test SynchronizedData."""
@@ -720,14 +743,6 @@ def test_synchronized_datas() -> None:
     updated_synchronized_data = synchronized_data_____.create()
     assert updated_synchronized_data.blacklisted_keepers == set()
 
-    # test wrong tx hashes serialization
-    synchronized_data_____.update(late_arriving_tx_hashes=["test"])
-    with pytest.raises(
-        ABCIAppInternalError,
-        match="internal error: Cannot parse late arriving hashes: test!",
-    ):
-        _ = synchronized_data_____.late_arriving_tx_hashes
-
 
 class TestResetRound(BaseCollectSameUntilThresholdRoundTest):
     """Test ResetRound."""
